
from jno.commands.command import Command

import os

class Init(Command):

	def run(self,argv,location):
		self.init_jno(location)

	def init_jno(self,jno_dir):
		lib_dir = os.path.join(jno_dir,'libraries')
		lib_dir_old = os.path.join(jno_dir,'lib')
		sketch_dir = os.path.join(jno_dir,'sketch')
		jno_file = os.path.join(jno_dir,'jno.jno')
		# create lib if does not exist
		if not os.path.exists(lib_dir):
			# check if we need to rename
			if os.path.isdir(lib_dir_old):
				os.rename(lib_dir_old, lib_dir)
			else:
				os.mkdir(lib_dir)
				# create putlibrarieshere.txt file, useful when using with git so directory gets committed
				with open(os.path.join(lib_dir,"putlibrarieshere.txt"),"wb") as libfile:
					pass

		# create sketch if does not exist
		if not os.path.exists(sketch_dir):
			os.mkdir(sketch_dir)
			# create sketch.ino in sketch dir with setup/loop functions
			with open(os.path.join(sketch_dir,"sketch.ino"),"wb") as sketchfile:
				sketchfile.write("void setup() {\n\n}\n")
				sketchfile.write("void loop() {\n\n}\n")

		# create jno.jno 
		if not os.path.exists(jno_file):
			with open(jno_file,'wb') as jno:
				jno.write("EXEC_DIR==NULL\n")
				jno.write("BOARD==uno\n")
				jno.write("BAUDRATE==9600\n")
				jno.write("PORT==DEFAULT\n")
				jno.write("SKETCH_DIR==DEFAULT\n")
		print('directories and .jno initialized')



from jno.util import parse_jno_file,valid_params
from jno.commands.command import Command

import getopt
import os
from sys import version_info

class SetDefault(Command):

	# Reads current .jno file, applies requested changes
	def run(self,argv,location):
		jno_dict = {}
		jno_dict = parse_jno_file(jno_dict,location)

		# get lowercase versions as well
		opt_arg_list = ['EXEC_DIR=','BAUDRATE=','PORT=','BOARD=']
		for n in range(0,len(opt_arg_list)):
			opt_arg_list.append(opt_arg_list[n].lower())

		try:
			opts,args = getopt.getopt(argv[1:],'',opt_arg_list)
		except getopt.GetoptError:
			print('invalid arguments')
			quit()
		
		for opt, arg in opts:
			opt = opt[2:].upper()
			if opt in valid_params:
				jno_dict[opt] = arg
				print('New default {0}: {1}'.format(opt,arg))

		self.create_new_jno_file(jno_dict,location)

	# Overwrites current .jno file
	def create_new_jno_file(self,jno_dict,location):
		with open(os.path.join(location,'jno.jno'),'w') as jno:
			if version_info < (3,0): # python2 code
				for key,value in jno_dict.iteritems():
					jno.write('{0}=={1}\n'.format(key,value))
			else: # python3 code
				for key,value in jno_dict.items():
					jno.write('{0}=={1}\n'.format(key,value))


from jno.util import interpret_configs
from jno.util import run_arduino_process
from jno.util import create_build_directory
from jno.util import get_common_parameters
from jno.util import JnoException
from jno.commands.command import Command

import getopt

class Upload(Command):

	def run(self,argv,__location__):
		jno_dict = interpret_configs(__location__)
		create_build_directory(jno_dict)
		arg_list = self.perform_upload(argv[1:],jno_dict)
		run_arduino_process(arg_list)

	# Create argument list for arduino build 
	def perform_upload(self,argv,jno_dict):
		# assemble command query
		# GOAL: <arduino exec> --upload <script> --board <board> --port <serial>
		arg_list = [jno_dict["EXEC_SCRIPT"]]
		# add common params - set pref 
		arg_list.extend(get_common_parameters(jno_dict))
		# add upload params
		arg_list.append("--upload")
		arg_list.append(jno_dict["SKETCH_INO"])
	
		try:
			opts,args = getopt.getopt(argv, 'p:',['board=','port='])
		except getopt.GetoptError:
			raise JnoException("invalid arguments")
		for opt, arg in opts:
			if opt in ("--board"):
				jno_dict["BOARD"] = arg.strip()
			elif opt in ("-p","--port"):
				jno_dict["PORT"] = arg.strip()
			elif opt in ("--verbose"):
				arg_list.append("--verbose")

		# add board params
		arg_list.append("--board")
		arg_list.append(self.formatBoard(jno_dict["BOARD"],jno_dict))
		# add port params
		arg_list.append("--port")
		arg_list.append(jno_dict["PORT"])

		return arg_list



from jno.util import interpret_configs
from jno.util import clean_directory
from jno.commands.command import Command

import os
from colorama import Fore

class Clean(Command):

	def run(self,argv,__location__):
		jno_dict = interpret_configs(__location__)
		if clean_directory(os.path.join(jno_dict["SKETCH_DIR"],".build")):
			print("{}Cleaned build files successfully{}".format(Fore.GREEN,Fore.RESET))
		else:
			print("{}Could not clean build files{}".format(Fore.RED,Fore.RESET))



import os
import re
import shutil
import subprocess
from shutil import rmtree
from collections import OrderedDict, namedtuple
from colorama import Fore


# Parameters to be included in .jno
valid_params = {
		'EXEC_DIR',
		'EXEC_LIBS',
		'BOARD',
		'PORT',
		'BAUDRATE',
		'SKETCH_DIR'
	}


# Exception class for jno-related exceptions
class JnoException(Exception):
	pass

# Replaces temporary dictionary values with actual values
def interpret_configs(__location__):
	jno_dict = read_configs(__location__)
	# check if EXEC_DIR exists
	if jno_dict["EXEC_DIR"] in ('NULL','DEFAULT'):
		raise JnoException("EXEC_DIR has not been initialized")
	# perform necessary additions/replacements
	if jno_dict["SKETCH_DIR"] == 'DEFAULT':
		jno_dict["SKETCH_DIR"] = os.getcwd()
	# if not absolute directory, make it local
	elif not jno_dict["SKETCH_DIR"].startswith('/'):
		jno_dict["SKETCH_DIR"] = os.path.join(os.getcwd(),jno_dict["SKETCH_DIR"])
	# create EXEC_SCRIPT; if on Windows, uses the better executable
	if os.name == 'nt':
		jno_dict["EXEC_SCRIPT"] = os.path.join(jno_dict["EXEC_DIR"],'arduino_debug')
	else:
		jno_dict["EXEC_SCRIPT"] = os.path.join(jno_dict["EXEC_DIR"],"arduino")
	# create SKETCH_INO
	jno_dict["SKETCH_INO"] = os.path.join(jno_dict["SKETCH_DIR"],'sketch/sketch.ino')
	# create SKETCH_LIBS
	jno_dict["SKETCH_LIBS"] = os.path.join(jno_dict["SKETCH_DIR"],'libraries')

	return jno_dict


# Parses global and local .jno, returning dictionary
def read_configs(__location__):
	jno_dict = {}
	jno_dict = parse_jno_file(jno_dict,__location__)
	if os.path.exists(os.path.join(os.getcwd(),'jno.jno')):
		jno_dict = parse_jno_file(jno_dict,os.getcwd())
	return jno_dict


# Parses .jno file in given directory, returning dictionary
def parse_jno_file(jno_dict,jno_dir):
	new_dict = {}

	with open(os.path.join(jno_dir,'jno.jno'),'r') as jno:
		for line in jno:
			line = line.strip()
			if len(line) == 0:
				continue
			param,conf_input = line.split('==')
			if param in valid_params:
				new_dict[param] = conf_input

	new_keys = new_dict.keys()
	for key in new_keys:
		if key in jno_dict:
			if new_dict[key] in ('NULL','DEFAULT'):
				continue
		jno_dict[key] = new_dict[key]
	return jno_dict


# Cleans selected directory
def clean_directory(dir_to_clean):
	# if exists, remove and replace
	if os.path.isdir(dir_to_clean):
		try:
			rmtree(dir_to_clean)
		except:
			return False
	# in either case, make the directory again
	try:
		os.mkdir(dir_to_clean)
	except:
		return False
	return True 


# Run arduino with an assembled argument list
def run_arduino_process(arg_list):
	try:
		returned = subprocess.check_call(arg_list)
	except subprocess.CalledProcessError as e:
		returned = e.returncode
	print(Fore.YELLOW + 'All Actions Complete: {}'.format(return_code_qualifier(returned)) + Fore.RESET)


# Returns meaning of return code
def return_code_qualifier(return_code):
	return_code_dict = {
		0:Fore.GREEN + "Success",
		1:Fore.RED + "Build failed or upload failed",
		2:Fore.RED + "Sketch not found",
		3:Fore.RED + "Invalid argument for commandline option",
		4:Fore.RED + "Preference passed to --get-pref does not exist"
	}
	return return_code_dict[return_code]

# Create directory for building
def create_build_directory(jno_dict):
	build_directory = os.path.join(jno_dict["SKETCH_DIR"],".build")
	if not os.path.isdir(build_directory):
		os.mkdir(build_directory)
	# while we are at it, check if library directory has the right name
	lib_dir = os.path.join(jno_dict["SKETCH_DIR"],"libraries")
	lib_dir_old = os.path.join(jno_dict["SKETCH_DIR"],"lib")
	if os.path.isdir(lib_dir_old):
		os.rename(lib_dir_old,lib_dir)

# Returns list of common parameters needed for upload/build
def get_common_parameters(jno_dict):
	# we are trying to set the build and sketchbook path
	build_path = os.path.join(jno_dict["SKETCH_DIR"],".build")
	sketchbook_path = jno_dict["SKETCH_DIR"]
	pref_string_list = []
	argument_list = []
	# fill out string list
	pref_string_list.append("build.path={}".format(build_path))
	pref_string_list.append("sketchbook.path={}".format(sketchbook_path))
	# fill out argument list
	for pref_string in pref_string_list:
		argument_list.append("--pref")
		argument_list.append(pref_string)
	# return arguments
	return argument_list

# Get and print list of all supported models
def get_all_models(jno_dict):
	# directores to ignore
	ignore_dirs = ["tools"]
	# get hardware directory
	arduino_hardware_dir = os.path.join(jno_dict["EXEC_DIR"],"hardware")
	# used to store all models
	all_models = []
	# do a walk
	directories = next(os.walk(arduino_hardware_dir))[1]
	for directory in directories:
		# if not an ignored dir, go into it
		if directory not in ignore_dirs:
			directory_path = os.path.join(arduino_hardware_dir,directory)
			subdirectories = next(os.walk(directory_path))[1]
			# in each directory here, go into it and do a get_boards... call
			for subdir in subdirectories:
				subdir_path = os.path.join(directory_path, subdir)
				path_prefix = "{}:{}:".format(directory,subdir)
				all_models.extend(get_boards_from_directory(subdir_path,path_prefix))
	#arduino_hardware_dir = os.path.join(jno_dict["EXEC_DIR"],"hardware/arduino/avr/")
	return all_models

# Returns model list from boards.txt in specified directory
def get_boards_from_directory(fileloc,prefix):
	# models is a list of Model classes
	models = []
	with open(os.path.join(fileloc,"boards.txt"),'r') as modelfile:

		still_expecting_menu_item_types = True
		
		# list for expected menu item types [(menu_item_type, menu_item_label), ...]
		expected_menu_item_types = []
		# keep track of current board info
		current_model = Model()
		
		for line in modelfile:
			# strip the line
			line = line.strip()
			# skip empty lines
			if len(line) == 0:
				continue
			# at the beginning of the file, we expect to see all the possible menu item types. Collect them
			if still_expecting_menu_item_types and current_model.board is None:
				menu_type_search_object = re.search("menu.[a-zA-Z0-9_-]*=(.*)", line)
				if menu_type_search_object is not None:
					# get menu item type + readable label
					expected_type_label,expected_readable_label = menu_type_search_object.group(0).split("=")
					expected_type_label = expected_type_label.split(".")[1]
					expected_menu_item_types.append((expected_type_label, expected_readable_label))
					continue
				
			# check if line depicts board name
			if ".name=" in line:
				# this is not a new menu item type line, so set expectation to False
				if still_expecting_menu_item_types:
					still_expecting_menu_item_types = False

				arduino_label,readable_label = line.strip().split(".name=")
				# check if we are on a different type of board now
				if current_model.board is not None and arduino_label != current_model.board:
					models.append(current_model)
					current_model = Model()
				# change the current labels
				current_model.board = arduino_label
				current_model.board_name = readable_label
				current_model.initialize_dict(expected_menu_item_types)
				current_model.set_prefix(prefix)

			# see if it is a different skew of current board type
			elif current_model.board is not None:
				search_object = re.search(current_model.board+".menu.[a-zA-Z0-9_-]*.[a-zA-Z0-9_-]*=(.*)", line)
				if search_object is not None:
					# figure out which menu type we got
					menu_item_type,menu_item_label = search_object.group(0).split("=")
					menu_item_type,menu_item_name = menu_item_type.split(".")[-2:]
					# add this menu_item_type + label to the proper item in current_menu_items
					current_model.add_menu_item(menu_item_type,menu_item_name,menu_item_label)

		# add last entry
		if current_model.board is not None:
			models.append(current_model)
			current_model = Model()

	return models


# A named tuple used to represent a specific menu item
MenuItem = namedtuple("MenuItem", ["name","label"])

# ModelData that is used to store label and all the subitems
class ModelData(object):
		def __init__(self, label):
			self.label = label
			self.items = []
			self.empty = True

		def is_empty(self):
			return self.empty

		def append(self, item):
			self.items.append(item)
			self.empty = False

		def get_first(self):
			if len(self.items) > 0:
				return self.items[0]
			else:
				return None

		def __str__(self):
			return "'ModelData with Label: {} and Items: {}'".format(self.label,self.items)

		def __repr__(self):
			return str(self)

# Model that stores data for a particular arduino board
class Model(object):

	def __init__(self):
		self.board = None
		self.board_name = None
		self.menu_item_dict = OrderedDict()
		self.empty = True
		self.argument_prefix = ""

	# initialize menu item dictionary
	def initialize_dict(self, expected_menu_list):
		for menu_item_type,menu_item_label in expected_menu_list:
			self.menu_item_dict[menu_item_type] = ModelData(menu_item_label)

	# check if there are no items added to the dictionary
	def is_empty(self):
		return self.empty

	# set and get prefix
	def set_prefix(self, prefix):
		self.argument_prefix = prefix

	def get_prefix(self):
		return self.argument_prefix

	# add a menu item for a particular type
	def add_menu_item(self, menu_item_type, menu_item_name, menu_item_label):
		self.menu_item_dict[menu_item_type].append(MenuItem(menu_item_name,menu_item_label))
		self.empty = False

	def __str__(self):
		return "'Model with board: {}, board_name: {}, and item dict: {}".format(self.board,self.board_name,self.menu_item_dict)

	def __repr__(self):
		return str(self)



#!/usr/bin/python2
from jno.util import get_all_models
from jno.util import JnoException
from colorama import Fore
from sys import version_info


class Command():

	def __init__(self,argv,location):
		self.run(argv,location)


	# Overload this function in actual commands
	def run(self,argv,location):
		pass


	# Return proper board formatting
	def formatBoard_old(self,board,jno_dict):
		prepared_board_string = None
		board_name = None
		all_models = get_all_models(jno_dict)
		if not board.startswith("arduino:avr:"):
			if ":" in board:
				prepared_board_string = board
				board_name = prepared_board_string.split(":")[2]
			else:
				prepared_board_string = "arduino:avr:{}".format(board)
		else:
			prepared_board_string = board
			board_name = prepared_board_string.split(":")[2]
		# Check if CPU was explicitly specified; if not, check if one can be given
		if ":cpu=" not in prepared_board_string:
			all_boards = get_all_models(jno_dict)
			for board_data in all_boards:
				# If board exists...
				if board_data[0] == board_name:
					cpu_names = board_data[2]
					# Add the first CPU option if available
					if len(cpu_names) > 0:
						prepared_board_string += ":cpu={}".format(cpu_names[0])
						print("{1}Using default {2}cpu={0}{3}".format(cpu_names[0],Fore.YELLOW,Fore.CYAN,Fore.RESET))
		return prepared_board_string

	def formatBoard(self,board_input,jno_dict):
		prepared_board_string = None
		model = None
		board = None
		param_list = []
		value_list = []
		# get all supported models
		all_models = get_all_models(jno_dict)
		
		# parse board string to separate board from possible paramters
		if ":" in board_input:
			# get board name separate from param_string
			try:
				board,param_string = board_input.split(":")
			except ValueError:
				raise JnoException("invalid board input; there should be max of 1 ':'")
			else:
				# get param and value pairs from raw string
				params_raw = param_string.split(",")
				for param_raw in params_raw:
					try:
						param,value = param_raw.split("=")
					except ValueError:
						raise JnoException("invalid board input; each parameter can only have 1 '='")
					else:
						param_list.append(param)
						value_list.append(value)
		# otherwise, the board name should havbe been provided
		else:
			board = board_input

		# get model
		for some_model in all_models:
			if some_model.board == board:
				model = some_model
				break
		# if not found, raise exception
		if model is None:
			raise JnoException("board '{}' not recognized".format(board))

		# figure out which params were provided; if not provided, choose default val
		if version_info < (3,0): # python2 code
			for param_type,data in model.menu_item_dict.iteritems():
				self.fill_out_param_value_lists(param_list,value_list,param_type,data)
		else: # python3 code
			for param_type,data in model.menu_item_dict.items():
				self.fill_out_param_value_lists(param_list,value_list,param_type,data)

		# if there are parameters to be input, account for that
		if len(param_list) > 0:
			prepared_board_string = model.get_prefix() + model.board + ":"
			string_params = []
			# combine param and value for strings
			for param,value in zip(param_list,value_list):
				string_params.append("{0}={1}".format(param,value))
				print("Setting {} to {}".format(param,value))
			prepared_board_string += ','.join(string_params)
		# otherwise, don't do anything with it
		else:
			prepared_board_string = model.get_prefix() + model.board
		# return prepared string
		return prepared_board_string

	# Helper function for formatBoard function
	def fill_out_param_value_lists(self,param_list,value_list,param_type,data):
		# if this param type doesn't have choices, skip it
		if data.is_empty():
			return
		# if this param was provided by the user, skip it
		if param_type in param_list:
			return
		# otherwise, choose default value for it
		param_list.append(param_type)
		value_list.append(data.get_first().name)



from jno.util import interpret_configs
from jno.commands.command import Command

import getopt
import serial
import threading
import sys
from colorama import Fore


# class used for serial communication
class JnoSerial(Command):

	def run(self,argv,__location__):
		jno_dict = interpret_configs(__location__)
		jno_dict = self.parse_serial_args(argv[1:],jno_dict)
		self.start_serialcomm(jno_dict)


	# Parse arguments passed into JnoSerial
	def parse_serial_args(self,argv,jno_dict):
		try:
			opts,args = getopt.getopt(argv, 'p:b:',['port=','baud='])
		except getopt.GetoptError:
			print(Fore.RED + 'invalid arguments' + Fore.RESET)
			quit()
		for opt, arg in opts:
			if opt in ("-p","--port"):
				if arg.strip() == "":
					raise ValueError("no port provided")
				jno_dict["PORT"] = arg
			elif opt in ("-b","--baud"):
				if arg.strip() == "":
					raise ValueError("no baudrate provided")
				jno_dict["BAUDRATE"] = arg
		return jno_dict


	# Open serial communication with arduino
	def start_serialcomm(self,jno_dict):
		PORT = jno_dict["PORT"]
		
		try:
			BAUD = int(jno_dict["BAUDRATE"])
			ard_serial = serial.Serial(PORT, BAUD)
		except ValueError as e:
			print(Fore.RED + str(e) + Fore.RESET)
			return
		except Exception as e:
			print(Fore.RED + str(e) + Fore.RESET)
			return
			
		if ard_serial.isOpen():
			ard_serial.close()
			ard_serial.open()

		print(Fore.CYAN + ':: successfully started serial')
		print(':: port: {1}{0}{2}'.format(PORT,Fore.YELLOW,Fore.CYAN))
		print(':: baudrate: {1}{0}{2}'.format(BAUD,Fore.YELLOW,Fore.CYAN))
		print(':: type {0}EXIT{1} to leave serial'.format(Fore.YELLOW,Fore.CYAN))
		print('' + Fore.RESET)
		# create thread for receiving serial stuff
		ser_event = threading.Event()
		ser_message_sent = threading.Event()
		ser_thread = threading.Thread(target=self.serial_function,args=(ard_serial,ser_event,ser_message_sent))
		ser_thread.daemon = True
		ser_thread.start()
		# echo user input to the serial device
		while True:
			user_inp = raw_input()
			ser_message_sent.set()
			if user_inp.strip() == "EXIT":
				break
			else:
				ard_serial.write(user_inp.strip())
		ser_event.set()


	# Function ran in another thread to read serial input
	def serial_function(self,ard_serial,ser_event,ser_message_sent):
		ard_serial.timeout = 0.1
		while not ser_event.is_set():
			recvd = ard_serial.read()
			sys.stdout.write(recvd)
			if not ser_message_sent.is_set() and recvd:
				print("")
				ser_message_sent.set()


	# Return first working serial connection
	def get_first_serial(self,baud):
		for num in range(0,5):
			pass


	# Try to start serial comm with device
	def get_serial_comm(self,port,baud):
		pass



#!/user/bin/env python
from setuptools import setup

#install requirements provided by requirements.txt
install_requirements = open("requirements.txt").read().split('\n')
#readme from repo
readme_contents = open("README.md").read()

jno_data = ['jno.jno']

jno_version = '0.4.0'

#input into setup
setup(
	name='jno',
	version=jno_version,
	description='Command line interface wrapper for Arduino IDE, inspired by ino',
	long_description=readme_contents,
	url = 'https://github.com/kosinkadink/jno',
	download_url = 'https://github.com/kosinkadink/jno/archive/{}.tar.gz'.format(jno_version),
	author='Jedrzej Kosinski',
	author_email='kosinkadink1@gmail.com',
	license='MIT',
	keywords='arduino interface wrapper',
	packages=['jno','jno.commands'],
	package_data={'jno':jno_data},
	install_requires=install_requirements,
	scripts=['bin/jno'],
	classifiers=[
		"Development Status :: 4 - Beta",
		"License :: OSI Approved :: MIT License",
		"Intended Audience :: Developers",
		"Operating System :: OS Independent",
		"Programming Language :: Python :: 2",
		"Programming Language :: Python :: 3",
		"Topic :: Software Development :: Embedded Systems"
	],
)


from jno.util import interpret_configs, get_all_models, JnoException
from jno.commands.command import Command

import os
from sys import version_info
import getopt
from colorama import Fore

class ListModels(Command):

	def run(self,argv,__location__):
		jno_dict = interpret_configs(__location__)
		models = get_all_models(jno_dict)
		self.perform_listmodels(argv[1:],models)

	def perform_listmodels(self,argv,models):
		is_board_provided = False
		try:
			opts,args = getopt.getopt(argv, '', ['board='])
		except getopt.GetoptError:
			raise JnoException("invalid arguments")
		# if a board is provided, show info about it
		for opt, arg in opts:
			if opt in ("--board"):
				is_board_provided = True
				self.printDataAboutModel(arg,models)
		# otherwise, list all models
		if not is_board_provided:
			self.printAllModels(models)

	def create_parameter_string(self,model):
		parameter_types = []
		parameter_string = ""

		if not model.is_empty():
			if version_info < (3,0): # python2 code
				for item_type,data in model.menu_item_dict.iteritems():
					if not data.is_empty():
						parameter_types.append(item_type)
			else: # python3 code
				for item_type,data in model.menu_item_dict.items():
					if not data.is_empty():
						parameter_types.append(item_type)

			parameter_string = ",".join(parameter_types)

			if len(parameter_string) > 0:
				parameter_string = "(" + parameter_string + ")"

		return parameter_string

	# Print each pair out nicely
	def printAllModels(self,models):
		print(Fore.CYAN + "")
		print("======================")
		print("SUPPORTED BOARD MODELS")
		print("----------------------")
		print("{3}{0}{4} :: {5}{1} {6}{2}{4}".format("Board","Board Name","(parameter types)",Fore.YELLOW,Fore.CYAN,Fore.MAGENTA,Fore.GREEN)) 
		print("----------------------")
		for model in models:
			# show relevant info, so see if any parameters exist for board
			parameter_string = self.create_parameter_string(model)
			print("{3}{0}{4} :: {5}{1} {6}{2}{4}".format(model.board,model.board_name,parameter_string,Fore.YELLOW,Fore.CYAN,Fore.MAGENTA,Fore.GREEN))
		# finish up the printing
		print("======================")
		print(""+Fore.RESET)

	def printDataAboutModel(self,board,models):
		# see if board exists
		relevant_model = None
		for model in models:
			if model.board == board:
				relevant_model = model
				break
		if relevant_model is None:
			raise JnoException("board '{}' is not recognized".format(board))

		print(Fore.CYAN + "")
		print("======================")
		print("Board: {2}{0}{4} ({1}){3}".format(relevant_model.board,relevant_model.board_name,Fore.YELLOW,Fore.CYAN,Fore.MAGENTA))
		print("  Prefix: {1}{0}{2}".format(relevant_model.get_prefix(),Fore.YELLOW,Fore.CYAN))
		# if model has relevant parameter values, show them
		if not relevant_model.is_empty():
			if version_info < (3,0): # python2 code
				for item_type,data in relevant_model.menu_item_dict.iteritems():
					self.print_parameters_with_values(item_type,data)
			else: # python3 code
				for item_type,data in relevant_model.menu_item_dict.items():
					self.print_parameters_with_values(item_type,data)
		print("======================")
		print(""+Fore.RESET)

	# Print out parameter with all possible values
	def print_parameters_with_values(self,item_type,data):
		# if this parameter has relevant values, show them
		if not data.is_empty():
			print("  Parameter: {2}{0}{3} ({1}){4}".format(item_type,data.label,Fore.GREEN,Fore.MAGENTA,Fore.CYAN))
			# parse through possible values
			for item in data.items:
				print("    {2}{0}{3} ({1}){4}".format(item.name,item.label,Fore.YELLOW,Fore.MAGENTA,Fore.CYAN))







#!/usr/bin/python2
import sys
import os
from colorama import init, Fore

import jno.commands
from jno.commands.setdefault import SetDefault
from jno.commands.init import Init
from jno.commands.jnoserial import JnoSerial
from jno.commands.build import Build
from jno.commands.upload import Upload
from jno.commands.listmodels import ListModels
from jno.commands.clean import Clean
from jno.util import JnoException

# directory from which this script is ran
__location__ = os.path.realpath(
	os.path.join(os.getcwd(), os.path.dirname(__file__)))
if os.name == 'nt':
	__location__ = __location__.replace('\\','/')
	init()


DEBUG = False


def init_command(argv):
	Init(argv,os.getcwd())

def build_command(argv):
	Build(argv,__location__)

def upload_command(argv):
	Upload(argv,__location__)

def serial_command(argv):
	JnoSerial(argv,__location__)

def setglobal_command(argv):
	SetDefault(argv,__location__)

def setlocal_command(argv):
	SetDefault(argv,os.getcwd())

def listmodels_command(argv):
	ListModels(argv,__location__)

def clean_command(argv):
	Clean(argv,__location__)


# Dictionary linking option to function
command_dict = {
	"init": init_command,
	"build": build_command,
	"upload": upload_command,
	"serial": serial_command,
	"setglobal": setglobal_command,
	"setlocal": setlocal_command,
	"listmodels": listmodels_command,
	"clean": clean_command
}


def main():
	args = sys.argv[1:]
	if len(args) == 0:
		print(Fore.RED + "No commands given" + Fore.RESET)
	else:
		command = args[0]
		try:
			jno_function = command_dict[command]
		except KeyError as e:
			print(Fore.RED + "ERROR: Command " + Fore.YELLOW + command + Fore.RED + " not found" + Fore.RESET)
		else:
			try:
				jno_function(args)
			except JnoException as e:
				print(Fore.RED + "ERROR: {}".format(str(e)) + Fore.RESET)

if __name__ == '__main__':
	main()




from jno.commands.init import Init
from jno.commands.build import Build
from jno.commands.upload import Upload
from jno.commands.jnoserial import JnoSerial
from jno.commands.setdefault import SetDefault
from jno.commands.listmodels import ListModels
